@@ -18,10 +18,11 @@ Processing configuration data in SSH (both client and server).
 #include "ssh2includes.h"
 #include "sshconfig.h"
 #include "sshuser.h"
-#include "userfile.h"
+#include "sshuserfile.h"
 #include "sshuserfiles.h"
 #include "sshcipherlist.h"
 #include "namelist.h"
+#include "sshdllist.h"
 
 #define SSH_DEBUG_MODULE "SshConfig"
 
@@ -83,11 +84,78 @@ void ssh_free_varsvals(int n, char **vars, char **vals)
   ssh_xfree(vals); 
 }
 
-/* Reads the host key that defined in the config data. 
-   Returns TRUE if successful. */
+
+/* Parses a given comma-separated list to tokens, which are stored in
+   a SshDlList. The list is allocated and returned by this
+   function. On error, returns NULL. */
+SshDlList ssh_config_parse_list(char *string,
+                                SshParameterValidityProc function,
+                                void *context)
+{
+  char *rest;
+  char *current;
+  SshDlList list;
+  
+  SSH_PRECOND(string != NULL);
+
+  list = ssh_dllist_allocate();
+
+  rest = string;
+
+  while (strlen(rest) != 0 && (current = ssh_name_list_get_name(rest)) != NULL)
+    {
+      char *temp;
+      int i, j;
+      
+      rest += strlen(current);
+      if (*rest == ',')
+        rest++;
+
+      /* strip whitespaces and non-printable characters */
+      temp = ssh_xcalloc(strlen(current) + 1, sizeof(char));
+      
+      for (i = 0, j = 0; i < strlen(current) ; i ++)
+        if(isascii(current[i]) && isprint(current[i]) && !isspace(current[i]))
+          {
+            temp[j] = current[i];
+            j++;
+          }
+      
+      temp[j] = '\0';
+      ssh_xfree(current);
+      current = temp;
+
+      /* If validity function is given, invoke it to check the current
+         parameter.*/
+      if (function)
+        if ((*function)(current, context))
+          {
+            ssh_xfree(current);
+            continue;
+          }
+      
+      if (ssh_dllist_add_item(list, (void *)current, SSH_DLLIST_END) !=
+          SSH_DLLIST_OK)
+        {
+          SSH_DEBUG(0, ("list operation gave error."));
+          ssh_dllist_rewind(list);
+          while (ssh_dllist_is_empty(list))
+            {
+              char *to_be_deleted;
+              to_be_deleted = ssh_dllist_delete_current(list);
+              ssh_xfree(to_be_deleted);
+            }
+
+          ssh_dllist_free(list);
+          
+          return NULL;
+        }      
+    }
+
+  return list;
+}
 
 /* Allocates and initializes a config structure */
-/* XXX for Windows */
 SshConfig ssh_config_init(Boolean client)
 {
   SshConfig config;
@@ -101,16 +169,23 @@ SshConfig ssh_config_init(Boolean client)
   config->callback_context = NULL;
 
   config->random_seed_file = ssh_xstrdup(SSH_RANDSEED_FILE);
+  config->pgp_public_key_file = ssh_xstrdup(SSH_PGP_PUBLIC_KEY_FILE);
+  config->pgp_secret_key_file = ssh_xstrdup(SSH_PGP_SECRET_KEY_FILE);
+
   config->forward_agent = TRUE;
   config->forward_x11 = TRUE;
-  config->password_authentication = TRUE;
+  config->password_authentication = -1;
   config->rhosts_authentication = TRUE;
   config->rhosts_pubkey_authentication = TRUE;
-  config->pubkey_authentication = TRUE;
+  config->pubkey_authentication = -1;
   config->force_ptty_allocation = FALSE;
   config->verbose_mode = FALSE;
   config->compression = FALSE;
 
+  config->allowed_authentications = NULL;
+  config->required_authentications = NULL;
+
+  config->user_known_hosts = TRUE;
   config->port = ssh_xstrdup("22");
   config->ciphers = NULL;
   config->user_conf_dir = ssh_xstrdup(SSH_USER_CONFIG_DIRECTORY);
@@ -120,10 +195,20 @@ SshConfig ssh_config_init(Boolean client)
   config->password_prompt = ssh_xstrdup("%U's password: ");
   config->password_guesses = 3;
 
+  config->max_connections = 0;
+  
   config->host_to_connect = NULL;
   config->login_as_user = NULL;
   config->local_forwards = NULL;
   config->remote_forwards = NULL;
+
+  config->allowed_hosts = NULL;
+  config->denied_hosts = NULL;
+  config->allowed_shosts = NULL;
+  config->denied_shosts = NULL;
+  config->require_reverse_mapping = FALSE;
+
+  config->log_facility = SSH_LOGFACILITY_AUTH;
   
   config->fall_back_to_rsh = TRUE;
   config->use_rsh = TRUE;
@@ -131,16 +216,20 @@ SshConfig ssh_config_init(Boolean client)
   config->strict_host_key_checking = FALSE;
   config->escape_char = ssh_xstrdup("~");
   config->go_background = FALSE;
-  config->use_nonpriviledged_port = FALSE;
   config->dont_read_stdin = FALSE;
-
-  config->permit_root_login = TRUE;
+  config->gateway_ports = FALSE;
+  
+  config->ignore_rhosts = FALSE;
+  config->ignore_root_rhosts = -1;
+  config->permit_root_login = SSH_ROOTLOGIN_TRUE;
   config->permit_empty_passwords = FALSE;
   config->strict_modes = TRUE;
   config->quiet_mode = FALSE;
   config->fascist_logging = FALSE;
   config->print_motd = TRUE;
+  config->check_mail = TRUE;
   config->keep_alive = TRUE;
+  config->no_delay = FALSE;
   config->listen_address = ssh_xstrdup("0.0.0.0");
   config->login_grace_time = 600;
   config->host_key_file = ssh_xstrdup(SSH_HOSTKEY_FILE);
@@ -159,18 +248,83 @@ SshConfig ssh_config_init(Boolean client)
   config->ssh1compatibility = FALSE;  
 #endif /* SSH1_COMPATIBILITY */
   config->ssh_agent_compat = SSH_AGENT_COMPAT_NONE;
-  
+
+  config->signer_path = ssh_xstrdup(SSH_SIGNER_PATH);
   return config;
 }
 
-/* Frees client configuration data. */
+/* This should be called after initializing the config-struct
+   (ie. after command-line variables have been parsed. This checks,
+   that some required members are initialized properly.*/
+void ssh_config_init_finalize(SshConfig config)
+{
+  /* Common. */
+  /* "hostbased"-authentication method is not enabled by default. */
+  if (!config->allowed_authentications)
+    ssh_config_parse_list((char *)SSH_AUTH_PASSWD "," SSH_AUTH_PUBKEY,
+                          NULL, NULL);
+  
+  /* Client. */
+  if (config->client)
+    {
+      /* Nothing here yet! */
+    }
+  /* Server. */
+  else
+    {
+      /* If IgnoreRootRhosts isn't defined at this stage, assign it to
+         the same as IgnoreRhosts. */
+      if (config->ignore_root_rhosts == -1)
+        config->ignore_root_rhosts = config->ignore_rhosts;
+    }  
+}
 
+/* Helper function, that destroys a list, and frees it's contents,
+   too. */
+void ssh_config_free_list(SshDlList list)
+{
+  char *to_be_deleted;
+
+  if (list == NULL)
+    return;
+  
+  ssh_dllist_rewind(list);
+  while (!ssh_dllist_is_empty(list))
+    {
+      to_be_deleted = ssh_dllist_delete_current(list);
+      ssh_xfree(to_be_deleted);
+    }
+    
+  ssh_dllist_free(list);
+}
+
+/* Helper function, that combines two lists. dst will receive nodes
+   from src, and src will be freed. (so the pointer isn't valid after
+   this call.) If one or both arguments are NULL, does nothing. */
+void ssh_config_combine_lists(SshDlList dst, SshDlList src)
+{
+  if (!dst || !src)
+    return;
+  
+  ssh_dllist_rewind(src);
+  while (!ssh_dllist_is_empty(src))
+    {
+      ssh_dllist_add_item(dst, ssh_dllist_delete_current(src), SSH_DLLIST_END);
+    }
+
+  ssh_dllist_free(src);
+}
+
+/* Frees client configuration data. */
 void ssh_config_free(SshConfig config)
 {
-  size_t i;
+  int i;
   
   /* free all allocated memory */
   ssh_xfree(config->random_seed_file);
+  ssh_xfree(config->pgp_public_key_file);
+  ssh_xfree(config->pgp_secret_key_file);
+
   ssh_xfree(config->port);
   ssh_xfree(config->ciphers);
   ssh_xfree(config->identity_file);
@@ -185,7 +339,12 @@ void ssh_config_free(SshConfig config)
   ssh_xfree(config->login_as_user);
   ssh_xfree(config->local_forwards);
   ssh_xfree(config->remote_forwards);
-
+  
+  ssh_config_free_list(config->allowed_hosts);
+  ssh_config_free_list(config->denied_hosts);
+  ssh_config_free_list(config->allowed_shosts);
+  ssh_config_free_list(config->denied_shosts);
+  
   ssh_xfree(config->forced_command);
 
   /* Free subsystem-strings */  
@@ -202,6 +361,7 @@ void ssh_config_free(SshConfig config)
       ssh_xfree(config->public_host_key_blob);
     }
 
+  ssh_xfree(config->signer_path);
   memset(config, 0, sizeof(*config));
   ssh_xfree(config);
 }
@@ -222,6 +382,42 @@ SshConfig ssh_client_create_config()
 }
 
 
+struct LogFacility
+{
+  SshLogFacility facility;
+  char *fac_name;
+} logfacilities[] =
+{
+  {SSH_LOGFACILITY_AUTH, "AUTH"},
+  {SSH_LOGFACILITY_SECURITY, "SECURITY"},
+  {SSH_LOGFACILITY_DAEMON, "DAEMON"},
+  {SSH_LOGFACILITY_USER, "USER"},
+  {SSH_LOGFACILITY_MAIL, "MAIL"},
+  {SSH_LOGFACILITY_LOCAL0, "LOCAL0"},
+  {SSH_LOGFACILITY_LOCAL1, "LOCAL1"},
+  {SSH_LOGFACILITY_LOCAL2, "LOCAL2"},
+  {SSH_LOGFACILITY_LOCAL3, "LOCAL3"},
+  {SSH_LOGFACILITY_LOCAL4, "LOCAL4"},
+  {SSH_LOGFACILITY_LOCAL5, "LOCAL5"},
+  {SSH_LOGFACILITY_LOCAL6, "LOCAL6"},
+  {SSH_LOGFACILITY_LOCAL7, "LOCAL7"},
+  {-1, NULL}
+};
+
+Boolean auth_param_validity(const char *param, void *context)
+{
+  if (strcasecmp(param, SSH_AUTH_PUBKEY) == 0)
+    return FALSE;
+
+  if (strcasecmp(param, SSH_AUTH_PASSWD) == 0)
+    return FALSE;
+
+  if (strcasecmp(param, SSH_AUTH_HOSTBASED) == 0)
+    return FALSE;
+
+  return TRUE;
+}
+
 /* Set the variable corresponding to `var' to `val' in config */
 
 Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
@@ -263,10 +459,68 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
       config->forward_x11 = bool;
       return FALSE;
     }
+
+  if (strcmp(var, "allowedauthentications") == 0)
+    {
+      if (config->password_authentication != -1 &&
+          (!config->client /*|| config->verbose_mode XXX not yet
+                             implemented in client*/))
+        ssh_warning("Defining AllowedAuthentications. Parameter "
+                    "PasswordAuthentication (already defined) will be "
+                    "ignored.");
+      
+      if (config->pubkey_authentication != -1 &&
+          (!config->client /*|| config->verbose_mode XXX not yet
+                             implemented in client*/))
+        ssh_warning("Defining AllowedAuthentications. Parameter "
+                    "PubkeyAuthentication (already defined) will be "
+                    "ignored.");
+        
+      ssh_config_free_list(config->allowed_authentications);
+
+      if ((config->allowed_authentications =
+           ssh_config_parse_list(val, auth_param_validity, NULL))
+          == NULL)
+        {
+          ssh_warning("Parsing of value for AllowedAuthentications failed.");
+          return TRUE;
+        }
+      else
+        {    
+          return FALSE;
+        }      
+    }
   
+  if (strcmp(var, "requiredauthentications") == 0)
+    {
+      ssh_config_free_list(config->required_authentications);
+
+      if ((config->required_authentications =
+           ssh_config_parse_list(val, auth_param_validity, NULL))
+          == NULL)
+        {
+          ssh_warning("Parsing of value for RequiredAuthentications failed.");
+          return TRUE;
+        }
+      else
+        {    
+          return FALSE;
+        }      
+    }
+
   if (strcmp(var, "passwordauthentication") == 0)
     {
+      if (config->allowed_authentications)
+        {
+          if (!config->client || config->verbose_mode)
+            ssh_warning("AllowedAuthentications is already defined, ignoring "
+                        "PasswordAuthentication keyword.");
+          return TRUE;
+        }
+      
       config->password_authentication = bool;
+      ssh_warning("PasswordAuthentication configuration keyword is "
+                  "deprecated. Use AllowedAuthentications.");
       return FALSE;
     }
   
@@ -286,7 +540,17 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
   if (strcmp(var, "pubkeyauthentication") == 0 ||
       strcmp(var, "rsaauthentication") == 0)
     {
+      if (config->allowed_authentications)
+        {
+          if (!config->client || config->verbose_mode)
+            ssh_warning("AllowedAuthentications is already defined, ignoring "
+                        "PubkeyAuthentication keyword.");
+          return TRUE;
+        }
+
       config->pubkey_authentication = bool;
+      ssh_warning("PubkeyAuthentication configuration keyword is "
+                  "deprecated. Use AllowedAuthentications.");
       return FALSE;
     }
   
@@ -398,6 +662,20 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
       return FALSE;
     }
 
+  if (strcmp(var, "pgppublickeyfile") == 0)
+    {
+      ssh_xfree(config->pgp_public_key_file);
+      config->pgp_public_key_file = ssh_xstrdup(val);
+      return FALSE;
+    }
+
+  if (strcmp(var, "pgpsecretkeyfile") == 0)
+    {
+      ssh_xfree(config->pgp_secret_key_file);
+      config->pgp_secret_key_file = ssh_xstrdup(val);
+      return FALSE;
+    }
+
   if (strcmp(var, "forcepttyallocation") == 0)
     {
       config->force_ptty_allocation = bool;
@@ -430,6 +708,12 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
       return FALSE;
     }
 
+  if (strcmp(var, "nodelay") == 0)
+    {
+      config->no_delay = bool;
+      return FALSE;
+    }
+
   if (strcmp(var, "ssh1compatibility") == 0)
     {
       config->ssh1compatibility = bool;
@@ -502,12 +786,6 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           return FALSE;
         }
       
-      if (strcmp(var, "usenonpriviledgedport") == 0)
-        {
-          config->use_nonpriviledged_port = bool;
-          return FALSE;
-        }
-      
       if (strcmp(var, "dontreadstdin") == 0)
         {
           config->dont_read_stdin = bool;
@@ -546,7 +824,6 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
             }
         }
 
-#ifndef SSHDIST_WINDOWS
       if (strcmp(var, "localforward") == 0)
         {
           if(ssh_parse_forward(&(config->local_forwards), val))
@@ -566,21 +843,156 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
             }
           return FALSE;
         }  
-#endif /* SSHDIST_WINDOWS */
+      if (strcmp(var, "sshsignerpath") == 0)
+        {
+          ssh_xfree(config->signer_path);
+          config->signer_path = ssh_xstrdup(val);
+          return FALSE;
+        }
+
+      if (strcmp(var, "gatewayports") == 0)
+        {
+          config->gateway_ports = bool;
+          return FALSE;
+        }
     }
   else
     {
       /* These parameters are only for the server */
+
+      if (strcmp(var, "allowhosts") == 0)
+        {
+          SshDlList temp;
+          
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for AllowHosts failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->allowed_hosts)
+                /* Allow the use of multiple AllowHosts directives. */
+                ssh_config_combine_lists(config->allowed_hosts, temp);
+              else
+                config->allowed_hosts = temp;
+
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "denyhosts") == 0)
+        {
+          SshDlList temp;
+          
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for DenyHosts failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->denied_hosts)
+                /* Allow the use of multiple DenyHosts directives. */
+                ssh_config_combine_lists(config->denied_hosts, temp);
+              else
+                config->denied_hosts = temp;
+                  
+              return FALSE;
+            }
+        }
+      
+      if (strcmp(var, "allowshosts") == 0)
+        {
+          SshDlList temp;
+          
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for AllowSHosts failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->allowed_hosts)
+                /* Allow the use of multiple AllowSHosts directives. */
+                ssh_config_combine_lists(config->allowed_shosts, temp);
+              else
+                config->allowed_shosts = temp;
+
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "denyshosts") == 0)
+        {
+          SshDlList temp;
+          
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for DenySHosts failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->denied_shosts)
+                /* Allow the use of multiple DenyHosts directives. */
+                ssh_config_combine_lists(config->denied_shosts, temp);
+              else
+                config->denied_shosts = temp;
+                  
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "requirereversemapping") == 0)
+        {
+          config->require_reverse_mapping = bool;
+          return FALSE;
+        }
+
+      if (strcmp(var, "userknownhosts") == 0)
+        {
+          config->user_known_hosts = bool;
+          return FALSE;
+        }
+      
+      if (strcmp(var, "syslogfacility") == 0)
+        {         
+          for (i = 0; logfacilities[i].fac_name != NULL; i++)
+            {
+              if (strcasecmp(logfacilities[i].fac_name, val) == 0)
+                {
+                  config->log_facility = logfacilities[i].facility;
+                  return FALSE;
+                }
+            }
+          ssh_warning("Unknown SyslogFacility \"%s\".", val);
+          return TRUE;
+        }
       
       if (strcmp(var, "ignorerhosts") == 0)
         {
           config->ignore_rhosts = bool;
           return FALSE;
         }
+
+      if (strcmp(var, "ignorerootrhosts") == 0)
+        {
+          config->ignore_root_rhosts = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "permitrootlogin") == 0)
         {
-          config->permit_root_login = bool;
+          if (strcmp(val, "nopwd") == 0)
+            config->permit_root_login = SSH_ROOTLOGIN_NOPWD;
+          else
+            config->permit_root_login = bool;
+          
           return FALSE;
         }
       
@@ -601,7 +1013,13 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           config->print_motd = bool;
           return FALSE;
         }
-      
+
+      if (strcmp(var, "checkmail") == 0)
+        {
+          config->check_mail = bool;
+          return FALSE;
+        }      
+
       if (strcmp(var, "listenaddress") == 0)
         {
           /* XXX some checks here */
@@ -643,7 +1061,7 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           
       if (strcmp(var, "logingracetime") == 0)
         {
-          if (num < 1)
+          if (num < 0)
             {
               ssh_warning("Ignoring illegal login grace time %d",
                           num);
@@ -659,15 +1077,22 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           return FALSE;
         }
 
+      if (strcmp(var, "maxconnections") == 0)
+        {
+          config->max_connections = num;
+          return FALSE;
+        }
+
       if (strcmp(var, "sshd1path") == 0)
         {
           ssh_xfree(config->ssh1_path);
           config->ssh1_path = ssh_xstrdup(val);
           return FALSE;
         }
+
+
       
       /* Parse subsystem definitions */
-      
       if (strncmp(var, SUBSYSTEM_PREFIX, SUBSYSTEM_PREFIX_LEN) == 0)
         {
           if (strlen(val) < 1)
