@@ -1,6 +1,6 @@
 /*
 
-  ssh2d.c
+  sshd2.c
   
   Authors:
         Tatu Ylonen <ylo@ssh.fi>
@@ -29,7 +29,8 @@
 #include "sshmsgs.h"
 #include "sigchld.h"
 #include "sshgetopt.h"
-
+#include "auths-common.h"
+#include "sshencode.h"
 #include <syslog.h>
 
 #ifdef HAVE_LIBWRAP
@@ -40,11 +41,18 @@
 #endif /* NEED_SYS_SYSLOG_H */
 int allow_severity = LOG_INFO;
 int deny_severity = LOG_WARNING;
-#endif /* LIBWRAP */
+#endif /* HAVE_LIBWRAP */
 
-#if HAVE_OSF1_C2_SECURITY
-#include "tcbc2.h"
-#endif /* HAVE_OSF1_C2_SECURITY */
+#ifdef HAVE_SIA
+#include "sshsia.h"
+#endif /* HAVE_SIA */
+
+#if HAVE_SCO_ETC_SHADOW
+#include <sys/types.h>
+#include <sys/security.h>
+#include <sys/audit.h>
+#include <prot.h>
+#endif /* HAVE_SCO_ETC_SHADOW */
 
 #define SSH_DEBUG_MODULE "Sshd2"
 
@@ -58,6 +66,7 @@ typedef struct SshServerData
   SshPrivateKey private_server_key;
   Boolean debug;
   SshTcpListener listener;
+  int connections;
   SshUser user;
   Boolean ssh_fatal_called;
 } *SshServerData;
@@ -75,77 +84,70 @@ void server_disconnect(int reason, const char *msg, void *context)
   switch(reason)
     {
     case SSH_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT:
-      ssh_log_event(SSH_LOGFACILITY_SECURITY, SSH_LOG_WARNING,
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_WARNING,
                     "Disallowed connect from denied host. '%s'",
                     msg);
       break;
     case SSH_DISCONNECT_PROTOCOL_ERROR:
-      if(c->shared->config->fascist_logging)
-        ssh_log_event(SSH_LOGFACILITY_SECURITY,
-                      SSH_LOG_WARNING,
-                      "Protocol error: '%s'", msg);
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_WARNING,
+                    "Protocol error: '%s'", msg);
       break;
     case SSH_DISCONNECT_KEY_EXCHANGE_FAILED:
-      if(c->shared->config->fascist_logging)
-        ssh_log_event(SSH_LOGFACILITY_AUTH,
-                      SSH_LOG_WARNING, 
-                      "Key exchange failed: '%s'", msg);
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_WARNING, 
+                    "Key exchange failed: '%s'", msg);
       break;
     case SSH_DISCONNECT_HOST_AUTHENTICATION_FAILED:
-      if(c->shared->config->fascist_logging)
-        ssh_log_event(SSH_LOGFACILITY_AUTH,
-                      SSH_LOG_WARNING,
-                      "Host authentication failed: '%s'", msg);
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_WARNING,
+                    "Host authentication failed: '%s'", msg);
       break;
     case SSH_DISCONNECT_MAC_ERROR:
-      if(c->shared->config->fascist_logging)
-        ssh_log_event(SSH_LOGFACILITY_AUTH,
-                      SSH_LOG_WARNING,
-                      "MAC failed, disconnecting: '%s'", msg);
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_WARNING,
+                    "MAC failed, disconnecting: '%s'", msg);
       break;
     case SSH_DISCONNECT_COMPRESSION_ERROR:
-      if(c->shared->config->fascist_logging)
-        ssh_log_event(SSH_LOGFACILITY_AUTH,
-                      SSH_LOG_WARNING,
-                      "compression error, disconnecting: '%s'", msg);
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_WARNING,
+                    "compression error, disconnecting: '%s'", msg);
       break;
     case SSH_DISCONNECT_SERVICE_NOT_AVAILABLE:
-      if(c->shared->config->fascist_logging)
-        ssh_log_event(SSH_LOGFACILITY_AUTH,
-                      SSH_LOG_WARNING,
-                      "service not available: '%s'", msg);
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_WARNING,
+                    "service not available: '%s'", msg);
       break;
     case SSH_DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED:
-      if(c->shared->config->fascist_logging)
-        ssh_log_event(SSH_LOGFACILITY_AUTH,
-                      SSH_LOG_INFORMATIONAL,
-                      "protocol version not supported: '%s'", msg);
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_INFORMATIONAL,
+                    "protocol version not supported: '%s'", msg);
       break;
     case SSH_DISCONNECT_HOST_KEY_NOT_VERIFIABLE:
-      if(c->shared->config->fascist_logging)
-        ssh_log_event(SSH_LOGFACILITY_AUTH,
-                      SSH_LOG_WARNING,
-                      "host key not verifiable: '%s'", msg);
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_WARNING,
+                    "host key not verifiable: '%s'", msg);
       break;
     case SSH_DISCONNECT_CONNECTION_LOST:
-      if(c->shared->config->fascist_logging)
-        ssh_log_event(SSH_LOGFACILITY_AUTH,
-                      SSH_LOG_INFORMATIONAL,
-                      "connection lost: '%s'", msg);
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_INFORMATIONAL,
+                    "connection lost: '%s'", msg);
       break;
     case SSH_DISCONNECT_BY_APPLICATION:
-      if(c->shared->config->fascist_logging)
-        ssh_log_event(SSH_LOGFACILITY_AUTH,
-                      SSH_LOG_INFORMATIONAL,
-                      "disconnected by application: '%s'", msg);        
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_INFORMATIONAL,
+                    "disconnected by application: '%s'", msg);        
       break;
     case SSH_DISCONNECT_AUTHENTICATION_ERROR:
-      ssh_log_event(SSH_LOGFACILITY_AUTH, SSH_LOG_WARNING,
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_WARNING,
                     "User authentication failed: '%s'",
                     msg);
       break;
     default:
-      ssh_log_event(SSH_LOGFACILITY_DAEMON, SSH_LOG_ERROR,
+      ssh_log_event(c->server->config->log_facility,
+                    SSH_LOG_ERROR,
                     "Unknown reason code for disconnect. msg: '%s'",
                     msg);
       ssh_debug("Unknown reason code for disconnect. msg: '%s'", msg);
@@ -206,7 +208,7 @@ void ssh_server_version_check(const char *version, void *context)
       strncmp(version, "SSH-1.99", 8) != 0 &&
       c->server->config->ssh1compatibility == TRUE &&
       c->server->config->ssh1_path != NULL &&
-      c->server->config->ssh1_args != NULL)
+      c->server->config->ssh1_argv != NULL)
     {
       ssh_debug("Executing %s for ssh1 compatibility.",
                 c->server->config->ssh1_path);
@@ -217,11 +219,11 @@ void ssh_server_version_check(const char *version, void *context)
       args[arg++] = "-V";
       snprintf(buf, sizeof(buf), "%s\n", version); /* add newline */
       args[arg++] = buf;
-      for (i = 1; c->server->config->ssh1_args[i]; i++)
+      for (i = 1; c->server->config->ssh1_argv[i]; i++)
         {
           if (arg >= sizeof(args)/sizeof(args[0]) - 2)
             ssh_fatal("Too many arguments for compatibility ssh1.");
-          aa = c->server->config->ssh1_args[i];
+          aa = c->server->config->ssh1_argv[i];
           if (strcmp(aa, "-f") == 0 ||
               strcmp(aa, "-b") == 0 ||
               strcmp(aa, "-g") == 0 ||
@@ -230,14 +232,14 @@ void ssh_server_version_check(const char *version, void *context)
               strcmp(aa, "-p") == 0)
             {
               args[arg++] = aa;
-              if (c->server->config->ssh1_args[i + 1])
-                args[arg++] = c->server->config->ssh1_args[++i];
+              if (c->server->config->ssh1_argv[i + 1])
+                args[arg++] = c->server->config->ssh1_argv[++i];
             }
           else
             if (strcmp(aa, "-d") == 0)
               {
                 args[arg++] = aa;
-                if (c->server->config->ssh1_args[i + 1])
+                if (c->server->config->ssh1_argv[i + 1])
                   i++; /* Skip the level. */
               }
             else
@@ -265,8 +267,259 @@ void ssh_server_version_check(const char *version, void *context)
     }
 }
 
-/* This function is called whenever we receive a new connection. */
+/* This function will be used to see what authentications we can use
+   for the user, and to provide a common place to put access
+   control-information. */
+char *auth_policy_proc(const char *user,
+                       const char *service,
+                       const char *client_ip,
+                       const char *client_port,
+                       const char *completed_authentications,
+                       void *context)
+{
+  SshServer server = (SshServer)context;
+  SshConfig config = server->config;
+  SshUser uc = NULL;
+  char *cp;
+  
+  SSH_DEBUG(2, ("user '%s' service '%s' client_ip '%s' client_port '%s' "
+                "completed '%s'",
+                user, service, client_ip, client_port,
+                completed_authentications));
 
+  /* Check whether user's login is allowed */
+  /* XXX AllowUsers */
+  if (ssh_server_auth_check_user(&uc, user, server->config))
+    {
+      /* failure */
+      char *s = "login to account %.100s not allowed or account non-existent.";
+      
+      ssh_log_event(config->log_facility,
+                    SSH_LOG_WARNING,
+                    s,
+                    user);
+      SSH_TRACE(1, (s, user));
+      /* Reject the login if the user is not allowed to log in. */
+      return ssh_xstrdup("");
+    }
+  
+  /* Check whether logins from remote host are allowed */
+  if (ssh_server_auth_check_host(server->common))
+    {
+      char *s;
+      s = "Connection from %.100s denied. Authentication as user " \
+        "%.100s was attempted.";
+      
+      /* logins from remote host are not allowed. */
+      ssh_log_event(server->config->log_facility, SSH_LOG_WARNING,
+                    s, server->common->remote_host,
+                    ssh_user_name(uc));
+      SSH_TRACE(1, (s, server->common->remote_host, ssh_user_name(uc)));
+      
+      /* Reject the login if the user is not allowed to log in from
+         specified host. */
+      return ssh_xstrdup("");
+    }
+
+
+  /* Check, whether user has passed all required authentications*/
+  if (completed_authentications != NULL &&
+      strlen(completed_authentications) > 0)
+    {
+      SshDlList required = server->config->required_authentications;
+      
+      /* If some authentication method has been passed, and there is
+         no list for required authentications, the user is
+         authenticated. */
+      if (!required)
+        return NULL;
+      
+      /* Go trough the list to see if we find a match for every method
+         needed. */
+      ssh_dllist_rewind(required);
+
+      do {
+        char *current = NULL;
+        
+        if ((current = ssh_dllist_current(required)) == NULL)
+          continue;
+
+        if (strstr(completed_authentications, current) == NULL)
+          goto construct;
+        
+      } while (ssh_dllist_fw(required, 1) == SSH_DLLIST_OK);  
+
+      /* if all were found from completed_authentications, the user is
+         authenticated.*/
+      return NULL;
+    }
+
+  /* Otherwise, construct a list of the authentications that can continue.
+     All supported authentication methods are included in the list. */
+ construct:
+  {
+    SshDlList allowed, required;
+    SshBuffer buffer;
+
+    allowed = server->config->allowed_authentications;
+    required = server->config->required_authentications;
+      
+    ssh_buffer_init(&buffer);
+      
+    if (allowed)
+      {
+        Boolean first = TRUE;
+          
+        ssh_dllist_rewind(allowed);
+          
+        do {
+          char *current = NULL;
+            
+          if ((current = ssh_dllist_current(allowed)) == NULL)
+            continue;
+            
+          if (strstr(completed_authentications, current) == NULL)
+            {
+              if (required)
+                {
+                  Boolean found = FALSE;
+                    
+                  ssh_dllist_rewind(required);
+                    
+                  do {
+                    char *current_req = NULL;
+                  
+                    if ((current_req = ssh_dllist_current(required)) == NULL)
+                      continue;
+                  
+                    if (strcmp(current, current_req) != 0)
+                      {
+                        found = TRUE;
+                        break;
+                      }
+                  
+                  
+                  } while (ssh_dllist_fw(required, 1) == SSH_DLLIST_OK);
+
+                  if (!found)
+                    continue;
+                }
+                
+              if (!first)
+                {
+                  ssh_buffer_append(&buffer, (unsigned char *)",", 1);
+                }
+                
+              ssh_buffer_append(&buffer, (unsigned char *)current,
+                                strlen(current));
+
+              if (first)
+                first = FALSE;
+            }
+            
+        } while (ssh_dllist_fw(allowed, 1) == SSH_DLLIST_OK);  
+
+        ssh_buffer_append(&buffer, (unsigned char *)"\0", 1);
+      }
+    else
+      {    
+        /* If publickey authentication is denied in the configuration
+           file, deny it here too. */
+        if (server->config->pubkey_authentication == FALSE )
+          SSH_DEBUG(3, ("Public key authentication is denied."));
+        else
+          ssh_buffer_append(&buffer, (unsigned char *) SSH_AUTH_PUBKEY ",",
+                            strlen(SSH_AUTH_PUBKEY ","));
+              
+        /* If password authentication is denied in the configuration
+           file, deny it here too. */
+        if (config->password_authentication == FALSE )
+          SSH_DEBUG(3, ("Password authentication is denied."));
+        else
+          ssh_buffer_append(&buffer, (unsigned char *) SSH_AUTH_PASSWD ",\0",
+                            strlen(SSH_AUTH_PASSWD ",") + 1);
+          
+      }
+    cp = ssh_xstrdup(ssh_buffer_ptr(&buffer));
+    ssh_buffer_uninit(&buffer);
+  }
+  
+  SSH_DEBUG(2, ("output: %s", cp));
+
+  return cp;
+}
+
+/* Forward declaration. */
+void ssh_login_grace_time_exceeded(void *context);
+
+/* This is called, when we have an authenticated user ready to continue. */
+
+void client_authenticated(const char *user, void *context)
+{
+  SshServerConnection connection = (SshServerConnection) context;
+  SshCommon common = connection->server->common;
+
+  /* We unregister the (possible) grace time callback. */
+  ssh_cancel_timeouts(ssh_login_grace_time_exceeded, SSH_ALL_CONTEXTS);
+  
+  ssh_log_event(common->config->log_facility,
+                SSH_LOG_NOTICE,
+                "User %s, coming from %s, authenticated.",
+                user, common->remote_host);
+}
+
+
+/* Callback to handle the closing of connections in the "mother"
+   process */
+void child_returned(pid_t pid, int status, void *context)
+{
+  SshServerData data = (SshServerData) context;
+
+  SSH_DEBUG(2, ("Child with pid '%d' returned with status '%d'.", \
+                pid, status));
+
+  if (data->config->max_connections)
+    {
+      data->connections--;
+      
+      SSH_DEBUG(4, ("%d connections now open. (max %d)", \
+                    data->connections, data->config->max_connections));
+    }
+}
+
+/* This callback gets called, if LoginGraceTime is exceeded. */
+void ssh_login_grace_time_exceeded(void *context)
+{
+  SshServerConnection connection = (SshServerConnection) context;
+  char s[] = "LoginGraceTime exceeded.";
+  
+  ssh_log_event(connection->server->config->log_facility,
+                SSH_LOG_WARNING,
+                "%s", s);
+  SSH_DEBUG(0, ("%s", s));
+
+  /* We send disconnect, and exit. If LoginGraceTime is exceeded,
+     there might be some kind of DoS-attack going on. */
+  ssh_conn_send_disconnect(connection->server->common->conn,
+                           SSH_DISCONNECT_HOST_AUTHENTICATION_FAILED,
+                           "Login grace time exceeded.");
+
+  ssh_server_destroy(connection->server);
+
+  exit(0);
+}
+
+/* This callback is called, when a stream needs to be destroyed 
+   with a small callback. */
+void destroy_stream_callback(void *context)
+{
+  SshStream stream = (SshStream)context;
+
+  ssh_stream_destroy(stream);
+  return;
+}
+
+/* This function is called whenever we receive a new connection. */
 void new_connection_callback(SshIpError error, SshStream stream,
                              void *context)
 {
@@ -274,6 +527,7 @@ void new_connection_callback(SshIpError error, SshStream stream,
   SshServerConnection c;
   pid_t ret;
   const char *s;
+  char buf[256];
   
   if (error != SSH_IP_NEW_CONNECTION)
     {
@@ -281,7 +535,73 @@ void new_connection_callback(SshIpError error, SshStream stream,
       return;
     }
 
-  ssh_debug("new_connection_callback");
+  if (!ssh_tcp_get_remote_address(stream, buf, sizeof(buf)))
+    {
+      SSH_DEBUG(2, ("failed to fetch remote ip address."));
+      ssh_log_event(data->config->log_facility, SSH_LOG_WARNING,
+                    "failed to fetch remote ip address.");
+      snprintf(buf, sizeof(buf), "UNKNOWN");
+    }
+  
+  ssh_log_event(data->config->log_facility, SSH_LOG_INFORMATIONAL,
+                "connection from \"%s\"", buf);
+  
+  SSH_DEBUG(2, ("new_connection_callback"));
+
+  /* check for MaxConnections */
+
+  if (data->config->max_connections)
+    {
+      if (data->connections >= data->config->max_connections)
+        {
+          SshBuffer *buffer;
+
+          char error[] = "Too many connections.";
+          char lang[] = "en";
+          
+          buffer = ssh_buffer_allocate();
+          
+          /* Send disconnect, SSH_DISCONNECT_BY_APPLICATION */
+          /* Construct the packet. */
+          ssh_encode_buffer(buffer,
+                            /* SSH_MSG_DISCONNECT */
+                            SSH_FORMAT_CHAR,
+                            (unsigned int) SSH_MSG_DISCONNECT,
+                            /* uint32 reason code */
+                            SSH_FORMAT_UINT32, (SshUInt32)
+                            SSH_DISCONNECT_BY_APPLICATION,
+                            /* string description */
+                            SSH_FORMAT_UINT32_STR, error, strlen(error),
+                            /* string language tag */
+                            SSH_FORMAT_UINT32_STR, lang, strlen(lang),
+                            SSH_FORMAT_END);
+
+          ssh_stream_write(stream, ssh_buffer_ptr(buffer),
+                           ssh_buffer_len(buffer));
+          /* We destroy the stream with a little timeout in order
+             to give some time to the protocol to send the disconnect
+             message.  If the message can't be sent in this time window,
+             it's obvious that we are under some kind of DoS attack
+             and it's OK just to destroy the stream. */
+          ssh_register_timeout(0L, 20000L, 
+                               destroy_stream_callback, (void *)stream);
+
+          ssh_log_event(data->config->log_facility, SSH_LOG_WARNING,
+                        "Refusing connection from \"%s\". Too many " \
+                        "open connections (max %d, now open %d).",
+                        buf, data->config->max_connections,
+                        data->connections);
+
+          ssh_buffer_free(buffer);
+          /* return from this callback. */
+          return;
+        }
+    }
+
+  /* Set socket to nodelay mode if configuration suggests this. */
+  ssh_socket_set_nodelay(stream, data->config->no_delay);
+  /* Set socket to keepalive mode if configuration suggests this. */
+  ssh_socket_set_keepalive(stream, data->config->keep_alive);
 
   /* Fork to execute the new child, unless in debug mode. */
   if (data->debug)
@@ -291,7 +611,6 @@ void new_connection_callback(SshIpError error, SshStream stream,
   if (ret == 0)
     {
       /* Child. */
-
       /* Destroy the listener. */
       if (data->listener)
         ssh_tcp_destroy_listener(data->listener);
@@ -303,40 +622,48 @@ void new_connection_callback(SshIpError error, SshStream stream,
       data->config->ssh1_fd = ssh_stream_fd_get_readfd(stream);
       
 #ifdef HAVE_LIBWRAP
-  {
-    struct request_info req;
-    void *old_handler;
-    
-    old_handler = signal(SIGCHLD, SIG_DFL);
-                
-    request_init(&req, RQ_DAEMON, av0, RQ_FILE,
-                 ssh_stream_fd_get_readfd(stream), NULL);
-    fromhost(&req); /* validate client host info */
-    if (!hosts_access(&req))
       {
-        ssh_warning("Denied connection from %s by tcp wrappers.",
-                    eval_client(&req));
-        ssh_log_event(SSH_LOGFACILITY_SECURITY, SSH_LOG_WARNING,
-                      "Denied connection from %s by tcp wrappers.",
-                      eval_client(&req));
-        refuse(&req); /* If connection is not allowed, clean up and exit. */
-      }
+        struct request_info req;
+        void *old_handler;
+        
+        old_handler = signal(SIGCHLD, SIG_DFL);
+        /* XXX SIGALRM, SIGHUP */
+        
+        request_init(&req, RQ_DAEMON, av0, RQ_FILE,
+                     ssh_stream_fd_get_readfd(stream), NULL);
+        fromhost(&req); /* validate client host info */
+        if (!hosts_access(&req))
+          {
+            ssh_warning("Denied connection from %s by tcp wrappers.",
+                        eval_client(&req));
+            ssh_log_event(data->config->log_facility, SSH_LOG_WARNING,
+                          "Denied connection from %s by tcp wrappers.",
+                          eval_client(&req));
+            refuse(&req);/* If connection is not allowed, clean up and exit.*/
+          }
 
-    signal(SIGCHLD, old_handler);
+        signal(SIGCHLD, old_handler);
     
-  }
+      }
 #endif /* HAVE_LIBWRAP */
   
       /* Create a context structure for the connection. */
       c = ssh_xcalloc(1, sizeof(*c));
       c->shared = data;
+      SSH_TRACE(2, ("Wrapping stream with ssh_server_wrap..."));
       c->server = ssh_server_wrap(stream, data->config, data->random_state,
                                   data->private_server_key, server_disconnect,
                                   server_debug,
                                   (data->config->ssh1compatibility &&
                                    data->config->ssh1_path != NULL) ?
                                   ssh_server_version_check : NULL,
+                                  auth_policy_proc,
+                                  client_authenticated,
                                   (void *)c);
+      SSH_TRACE(2, ("done."));
+      if (data->config->login_grace_time > 0)
+        ssh_register_timeout((long)data->config->login_grace_time, 0L,
+                             ssh_login_grace_time_exceeded, c);
     }
   else
     {
@@ -345,7 +672,7 @@ void new_connection_callback(SshIpError error, SshStream stream,
         {
           s = "Forking a server for a new connection failed.";
           ssh_warning(s);
-          ssh_log_event(SSH_LOGFACILITY_DAEMON, SSH_LOG_WARNING, s);
+          ssh_log_event(data->config->log_facility, SSH_LOG_WARNING, s);
           ssh_stream_write(stream, (const unsigned char *)s, strlen(s));
           ssh_stream_write(stream, (const unsigned char *)"\r\n", 2);
         }
@@ -358,6 +685,21 @@ void new_connection_callback(SshIpError error, SshStream stream,
 
       /* Update the random seed file on disk. */
       ssh_randseed_update(data->user, data->random_state, data->config);
+
+      if (data->config->max_connections)
+        {
+          data->connections++;
+
+          SSH_DEBUG(4, ("Registering sigchld-handler for child '%d'.", \
+                        ret));
+
+          SSH_DEBUG(4, ("%d connections now open. (max %d)", \
+                        data->connections, data->config->max_connections));
+                        
+          ssh_sigchld_register(ret, child_returned,
+                               data);
+        }
+      
     }
 
   ssh_debug("new_connection_callback returning");
@@ -389,7 +731,7 @@ void server_ssh_fatal(const char *msg, void *context)
   SshServerData data = (SshServerData)context;
   data->ssh_fatal_called = TRUE;
 
-  ssh_log_event(SSH_LOGFACILITY_DAEMON, SSH_LOG_ERROR, "FATAL ERROR: %s", 
+  ssh_log_event(data->config->log_facility, SSH_LOG_ERROR, "FATAL ERROR: %s", 
                 msg);
 
   fprintf(stderr, "FATAL: %s\r\n", msg);  
@@ -430,6 +772,22 @@ int ssh_log_facility(SshLogFacility facility)
       return LOG_USER;
     case SSH_LOGFACILITY_MAIL:
       return LOG_MAIL;
+    case SSH_LOGFACILITY_LOCAL0:
+      return LOG_LOCAL0;
+    case SSH_LOGFACILITY_LOCAL1:
+      return LOG_LOCAL1;
+    case SSH_LOGFACILITY_LOCAL2:
+      return LOG_LOCAL2;
+    case SSH_LOGFACILITY_LOCAL3:
+      return LOG_LOCAL3;
+    case SSH_LOGFACILITY_LOCAL4:
+      return LOG_LOCAL4;
+    case SSH_LOGFACILITY_LOCAL5:
+      return LOG_LOCAL5;
+    case SSH_LOGFACILITY_LOCAL6:
+      return LOG_LOCAL6;
+    case SSH_LOGFACILITY_LOCAL7:
+      return LOG_LOCAL7;      
     }
   ssh_debug("ssh_log_facility: Unknown facility.");
   return -1;
@@ -454,7 +812,7 @@ void server_ssh_log(SshLogFacility facility, SshLogSeverity
       if (config->verbose_mode)
         logopt |= LOG_PERROR;
 #endif /* LOG_PERROR */
-      logfac = LOG_DAEMON;
+      logfac = ssh_log_facility(config->log_facility);
 
       openlog(av0, logopt, logfac);
       logopen = 1;
@@ -496,6 +854,30 @@ Boolean parameter_defined(const char param, int num, char **elements)
   return FALSE;
 }
 
+Boolean restart;
+
+/* signal callback for SIGHUP*/
+
+void sighup_handler(int signal, void *context)
+{
+  SshServerData data = (SshServerData) context;
+  
+  if (signal != SIGHUP)
+    {
+      SSH_DEBUG(0, ("Invalid signal received by SIGHUP-handler."));
+      ssh_log_event(data->config->log_facility, SSH_LOG_WARNING,
+                    "Invalid signal received by SIGHUP-handler.");
+      return;
+    }
+
+  /* We cannot call fork() and exec() here directly, because we are in
+     a signal handler. It seems that eventloop must be uninitialized
+     for this to work. */
+  restart = TRUE;
+
+  ssh_event_loop_abort();
+}
+
 /*
  *
  *  SSH2 server main()
@@ -504,7 +886,7 @@ Boolean parameter_defined(const char param, int num, char **elements)
 
 int main(int argc, char **argv)
 {
-  int i;
+  int i = 0;
   SshServerData data;
   SshUser user;
   char config_fn[1024];
@@ -518,17 +900,23 @@ int main(int argc, char **argv)
     av0 = argv[0];
 
   /* Initializations */
-
-#if HAVE_OSF1_C2_SECURITY
-  /* this is still very heavily under construction. */
-  tcbc2_initialize_security(argc, argv);
-#endif /* HAVE_OSF1_C2_SECURITY */
+  restart = FALSE;
   
+#if HAVE_SCO_ETC_SHADOW
+  set_auth_parameters(argc, argv);
+#endif /* HAVE_SCO_ETC_SHADOW */
+
+#if HAVE_SIA
+  initialize_sia(argc, argv);
+#endif /* HAVE_SIA */
+
   data = ssh_xcalloc(1, sizeof(*data));
   user = ssh_user_initialize(NULL, TRUE);
   
   data->ssh_fatal_called = FALSE;
 
+  data->connections = 0;
+  
   /* Create config context. */
   data->config = ssh_server_create_config();
 
@@ -558,8 +946,9 @@ int main(int argc, char **argv)
   ssh_event_loop_initialize();
   
   /* Save command line options for ssh1 compatibility code. */
-  data->config->ssh1_args = argv;
-
+  data->config->ssh1_argv = argv;
+  data->config->ssh1_argc = argc;
+  
   /* Save information about current user. */
   data->user = user;
   
@@ -567,6 +956,9 @@ int main(int argc, char **argv)
   ssh_signals_prevent_core(TRUE, data);
   ssh_register_signal(SIGPIPE, NULL, NULL);
 
+  /* register SIGHUP for restart callback */
+  ssh_register_signal(SIGHUP, sighup_handler, data);
+  
   /* Register SIGCHLD signal handler, to kill those darn zombies */
 
   ssh_sigchld_initialize();
@@ -575,8 +967,20 @@ int main(int argc, char **argv)
      on the commandline. */
   if (!parameter_defined('f', argc, argv))
     {
+      char *conf_dir;
+
+      if (ssh_user_uid(user) == 0 )
+        {
+          conf_dir = ssh_xstrdup(SSH_SERVER_DIR);
+        }
+      else
+        {
+          if ((conf_dir = ssh_userdir(user, data->config, TRUE)) == NULL)
+            ssh_fatal("no ssh2 user directory");
+        }
+
       snprintf(config_fn, sizeof(config_fn), "%s/%s",
-               ssh_userdir(user, data->config, TRUE), SSH_SERVER_CONFIG_FILE);
+               conf_dir, SSH_SERVER_CONFIG_FILE);
       if (!ssh_config_read_file(user, data->config, NULL, config_fn, NULL))
         ssh_warning("%s: Failed to read config file %s", av0, config_fn);
     }
@@ -616,7 +1020,7 @@ int main(int argc, char **argv)
           ssh_debug_set_level_string("2");
           break;
 
-              /* An additional configuration file */
+          /* An additional configuration file */
         case 'f':
           if (!ssh_optval)
             ssh_fatal("%s: Illegal -f parameter.", av0);
@@ -656,7 +1060,7 @@ int main(int argc, char **argv)
           i++;
           break;
 
-              /* is inetd enabled ? */
+          /* is inetd enabled ? */
         case 'i':
           data->config->inetd_mode = (ssh_optval != 0);
           break;
@@ -679,7 +1083,7 @@ int main(int argc, char **argv)
           i++;
           break;
 
-              /* Quiet mode */
+          /* Quiet mode */
         case 'q':
           data->config->quiet_mode = (ssh_optval != 0);
           break;
@@ -715,11 +1119,14 @@ int main(int argc, char **argv)
   data->random_state = ssh_randseed_open(user, data->config);
 
 #if 0
-  /* read the server key (if needed) */ 
+  /* generate the server key (if needed) */ 
   data->private_server_key = generate_server_key(data->config, 
                                                  data->random_state);
 #endif /* 0 */
- 
+
+  /* Finalize the initialization. */
+  ssh_config_init_finalize(data->config);
+
   ssh_debug("Becoming server.");
   
   /* Check if we are being called from inetd. */
@@ -727,6 +1134,9 @@ int main(int argc, char **argv)
     {
       SshStream stream;
 
+      ssh_log_event(data->config->log_facility,
+                    SSH_LOG_WARNING,
+                    "Starting daemon in inetd mode.");
       /* We are being called from inetd.  Take stdio to be the connection
          and proceed with the new connection. */
       stream = ssh_stream_fd_stdio();
@@ -748,6 +1158,11 @@ int main(int argc, char **argv)
                   data->config->port);
       ssh_debug("Listener created");
 
+      ssh_log_event(data->config->log_facility,
+                    SSH_LOG_WARNING,
+                    "Listener created on port %s.",
+                    data->config->port);
+      
       /* If not debugging, fork into background. */
       if (!data->debug)
         {
@@ -784,7 +1199,7 @@ int main(int argc, char **argv)
           setpgrp(0, 0);
 #else /* ultrix */
           if (setsid() < 0)
-            ssh_log_event(SSH_LOGFACILITY_DAEMON, SSH_LOG_NOTICE,
+            ssh_log_event(data->config->log_facility, SSH_LOG_NOTICE,
                           "setsid: %.100s", strerror(errno));
 #endif /* ultrix */
 #endif /* HAVE_SETSID */
@@ -810,6 +1225,10 @@ int main(int argc, char **argv)
       fprintf(f, "%ld\n", (long)getpid());
       fclose(f);
     }
+
+  ssh_log_event(data->config->log_facility,
+                SSH_LOG_WARNING,
+                "Daemon is running.");
   
   ssh_debug("Running event loop");
   ssh_event_loop_run();
@@ -820,7 +1239,45 @@ int main(int argc, char **argv)
   ssh_event_loop_uninitialize();
 
   if (data->listener)
-    remove(pidfile);
+    {
+      remove(pidfile);
+      
+      if (restart)
+        {
+          int ret;
+          
+          ssh_tcp_destroy_listener(data->listener);
+          data->listener = NULL;
+          
+          SSH_DEBUG(0, ("restarting..."));
+          ssh_log_event(data->config->log_facility, SSH_LOG_WARNING,
+                        "restarting...");
+          
+          ret = fork();
+          
+          if (ret == 0)
+            {
+              /* Child */
+              execv(argv[0], argv);
+              ssh_fatal("Restart (exec) failed on SIGHUP. "
+                        "(error message \"%s\")",
+                        strerror(errno));
+            }
+          else
+            {
+              /* Parent */
+              if (ret == -1)
+                {
+                  /* Fork failed */
+                  ssh_fatal("Restart (fork) failed on SIGHUP.");
+                }
+              else
+                {
+                  exit(0);
+                }
+            }
+        }
+    }
   
   return 0;
 }
