@@ -329,7 +329,7 @@ SshFileClientRequest ssh_file_request(SshFileClient client,
   va_start(va, reply_type);
   ssh_buffer_init(&buffer);
   ssh_encode_buffer(&buffer,
-                    SSH_FORMAT_UINT32, (unsigned long)client->next_id,
+                    SSH_FORMAT_UINT32, client->next_id,
                     SSH_FORMAT_END);
   ssh_encode_va(&buffer, va);
 
@@ -409,7 +409,7 @@ void ssh_file_client_open(SshFileClient client,
   
   request = ssh_file_request(client, SSH_FXP_OPEN, SSH_FILEXFER_HANDLE_REPLY,
                              SSH_FORMAT_UINT32_STR, name, strlen(name),
-                             SSH_FORMAT_UINT32, (unsigned long)pflags,
+                             SSH_FORMAT_UINT32, (SshUInt32) pflags,
                              SSH_FORMAT_EXTENDED, 
                                ssh_file_attrs_encoder, attributes,
                              SSH_FORMAT_END);
@@ -428,7 +428,8 @@ void ssh_file_client_read(SshFileHandle handle,
                           SshFileDataCallback callback,
                           void *context)
 {
-  unsigned long off_high, off_low;
+  SshUInt64 seek_offset = 0L;
+  
   SshFileClientRequest request;
   
   if (handle->client->eof_received)
@@ -437,14 +438,12 @@ void ssh_file_client_read(SshFileHandle handle,
       return;
     }
 
-  off_low = offset & 0xffffffffL;
-  off_high = sizeof(offset) > 4 ? (offset >> 32) : 0;
+  seek_offset = (SshUInt64)offset;
   request = ssh_file_request(handle->client, SSH_FXP_READ,
                              SSH_FILEXFER_DATA_REPLY,
                              SSH_FORMAT_UINT32_STR, handle->value, handle->len,
-                             SSH_FORMAT_UINT32, off_high,
-                             SSH_FORMAT_UINT32, off_low,
-                             SSH_FORMAT_UINT32, (unsigned long)len,
+                             SSH_FORMAT_UINT64, seek_offset,
+                             SSH_FORMAT_UINT32, (SshUInt32) len,
                              SSH_FORMAT_END);
   request->data_callback = callback;
   request->context = context;
@@ -459,7 +458,7 @@ void ssh_file_client_write(SshFileHandle handle,
                            SshFileStatusCallback callback,
                            void *context)
 {
-  unsigned long off_high, off_low;
+  SshUInt64 seek_offset = 0L;
   SshFileClientRequest request;
   
   if (handle->client->eof_received)
@@ -468,14 +467,12 @@ void ssh_file_client_write(SshFileHandle handle,
       return;
     }
 
-  off_low = offset & 0xffffffffL;
-  off_high = sizeof(offset) > 4 ? (offset >> 32) : 0;
+  seek_offset = (SshUInt64)offset;
   request = ssh_file_request(handle->client, SSH_FXP_WRITE,
                              SSH_FILEXFER_STATUS_REPLY,
                              SSH_FORMAT_UINT32_STR, 
                                handle->value, handle->len,
-                             SSH_FORMAT_UINT32, off_high,
-                             SSH_FORMAT_UINT32, off_low,
+                             SSH_FORMAT_UINT64, seek_offset,
                              SSH_FORMAT_UINT32_STR, 
                                buf, len,
                              SSH_FORMAT_END);
@@ -824,7 +821,7 @@ void ssh_file_client_receive_proc(SshPacketType type,
   SshFileClientRequest request;
   SshFileHandle handle;
   size_t bytes, slen, offset;
-  unsigned long u, id;
+  SshUInt32 u, id;
   unsigned int i, a;
   unsigned char *s, *name, *long_name;
   SshFileAttributes attrs;
@@ -1176,7 +1173,8 @@ SshFileClient ssh_file_client_wrap(SshStream stream)
 
   /* Send initialization packet. */
   ssh_packet_wrapper_send_encode(client->conn, SSH_FXP_INIT,
-                                 SSH_FORMAT_UINT32, 0L, SSH_FORMAT_END);
+                                 SSH_FORMAT_UINT32, (SshUInt32) 0,
+                                 SSH_FORMAT_END);
 
   return client;
 }
